<%
   Class ColumnInformation
      '************** MEMBER VARIABLES *********************
      Private strName
      Private iDataType
      Private bolDefault
      Private strDefault
      Private bolNullable
      Private strDescription
      Private varValue
      Private bolIdentity
      Private bolPrimaryKey
      '*****************************************************

      '****************** GET PROPERTIES *******************
      Public Property Get Identity()
        Identity = bolIdentity
      End Property

      Public Property Get PrimaryKey()
        PrimaryKey = bolPrimaryKey
      End Property

      Public Property Get Name()
        Name = strName
      End Property

      Public Property Get DataType()
        DataType = iDataType
      End Property

      Public Property Get HasDefault()
        HasDefault = bolDefault
      End Property

      Public Property Get DefaultValue()
        DefaultValue = strDefault
      End Property

      Public Property Get Nullable()
        Nullable = bolNullable
      End Property

      Public Property Get Description()
        Description = strDescription
      End Property

      Public Property Get Value()
        Value = varValue
      End Property
      '*****************************************************

      '****************** LET PROPERTIES *******************
      Public Property Let Identity(bolIdent)
        bolIdentity = bolIdent
      End Property

      Public Property Let PrimaryKey(bolPK)
        bolPrimaryKey = bolPK
      End Property

      Public Property Let Name(str)
        strName = str
      End Property

      Public Property Let DataType(iValue)
        iDataType = iValue
      End Property

      Public Property Let HasDefault(bolHasDefault)
        bolDefault = bolHasDefault
      End Property

      Public Property Let DefaultValue(str)
        strDefault = str
      End Property

      Public Property Let Nullable(bolIsNullable)
        bolNullable = bolIsNullable
      End Property

      Public Property Let Description(str)
        strDescription = str
      End Property

      Public Property Let Value(var)
        varValue = var
      End Property
      '*****************************************************
   End Class


   Class Columns
      '************** MEMBER VARIABLES *********************
      Private objColumnDict
      '*****************************************************

      '******************* EVENT HANDLERS ******************
      Private Sub Class_Initialize()
        Set objColumnDict = Server.CreateObject("Scripting.Dictionary")
      End Sub

      Private Sub Class_Terminate()
        Set objColumnDict = Nothing        'Clean up...
      End Sub
      '*****************************************************

      
      '********************** METHODS **********************
      Public Sub PopulateColumns(strTableName, objConn, strWhereClause)
        'This sub is responsible for populating the objColumnDict
        'dictionary object with the columns from the table strTableName
        'objConn is expected to be an opened connection to the
        'proper database

        'Make sure a proper table name was passed in
        If Len(strTableName) = 0 then
           Err.Raise vbObjectError + 1280, "Columns Class", _
              "In PopulateColumns method: TableName not supplied."
        End If

        Dim objRS     'Open up the column schema
        Set objRS = objConn.OpenSchema(adSchemaColumns, _
                             Array(Empty, Empty, strTableName, Empty))

        'Add a ColumnInformation object instance for each column that
        'exists within the table specified by strTableName.  Also,
        'build up a SQL statement
        Dim objColumn, strSQL
        strSQL = "SELECT "
        Do While Not objRS.EOF
           Set objColumn = New ColumnInformation

           objColumn.Name = objRS("COLUMN_NAME")
           objColumn.DataType = objRS("DATA_TYPE")
           objColumn.HasDefault = objRS("COLUMN_HASDEFAULT")
           objColumn.DefaultValue = objRS("COLUMN_DEFAULT")
           objColumn.Nullable = objRS("IS_NULLABLE")
           objColumn.Description = objRS("DESCRIPTION")
           objColumn.Identity = False
           objColumn.PrimaryKey = False

           objColumnDict.Add objColumn.Name, objColumn

           strSQL = strSQL & "[" & objColumn.Name & "]"

           Set objColumn = Nothing
           objRS.MoveNext

           If Not objRS.EOF then strSQL = RTrim(strSQL) & ","
        Loop

        'Now, we need to grab the PrimaryKey information
        objRS.Close
        Set objRS = Nothing

        Set objRS = objConn.OpenSchema(adSchemaPrimaryKeys, _
                           Array(Empty, Empty, strTableName))

        Do While Not objRS.EOF
           objColumnDict(objRS("COLUMN_NAME").Value).PrimaryKey = True
           objRS.MoveNext
        Loop
           

        'Now we need to obtain the identity columns/column values
        strSQL = strSQL & " FROM [" & strTableName & "]"
        If Len(strWhereClause) > 0 then
           strSQL = strSQL & " WHERE " & strWhereClause
        End If

        'We need to grab a row from the database to determine if we have any 
        'identity columns and to populate our row values, if needed
        objRS.Close
        Set objRS = Nothing         
        
        Set objRS = Server.CreateObject("ADODB.Recordset")
        objRS.MaxRecords = 1    'Get only one record
        objRS.Open strSQL, objConn

        'If there are no records in this table, we can't perform this loop
        If Not objRS.EOF then
          Dim objField
          For Each objField in objRS.Fields
            objColumnDict(objField.Name).Value = objField.Value

            If objField.Properties("ISAUTOINCREMENT") then
              objColumnDict(objField.Name).Identity = True
            End If
          Next
        End If


        'Clean up...
        objRS.Close
        Set objRS = Nothing         
      End Sub


      Public Function GetColumnInformation(strColumnName)
         'Returns a specific column's information... that is, return a
         'reference to an instance of a ColumnInformation object
         Set GetColumnInformation = objColumnDict(strColumnName)
      End Function


      Public Function GetPrimaryKeys()
         Dim strColumnName, strResult
         For Each strColumnName in objColumnDict
           If objColumnDict(strColumnName).PrimaryKey then
             If Len(strResult) > 0 then strResult = strResult & " AND "
             strResult = strResult & "[" & _
                      objColumnDict(strColumnName).Name & _
                      "] = " & QuotedValue(strColumnName)

             'Make sure we got a valid QuotedValue
             If Len(QuotedValue(strColumnName)) = 0 then
                Err.Raise vbObjectError + 1281, "Columns Class", _
                     "In method GetPrimaryKeys: Unable to Retrieve Value"
             End If
           End If
         Next

         GetPrimaryKeys = strResult
      End Function


      Public Function GetCurrentPrimaryKeys(objRS)
         Dim strColumnName, strResult
         For Each strColumnName in objColumnDict
           If objColumnDict(strColumnName).PrimaryKey then
             If Len(strResult) > 0 then strResult = strResult & " AND "
             strResult = strResult & "[" & _
                      objColumnDict(strColumnName).Name & _
                      "] = " & CurrentQuotedValue(strColumnName, objRS)

             'Make sure we got a valid QuotedValue
             If Len(CurrentQuotedValue(strColumnName, objRS)) = 0 then
                Err.Raise vbObjectError + 1281, "Columns Class", _
                     "In method GetCurrentPrimaryKeys: Unable to Retrieve Value"
             End If
           End If
         Next

         GetCurrentPrimaryKeys = strResult
      End Function


      Private Function CurrentQuotedValue(strColumnName, objRS)
         'Returns the value of a column, surrounding it with
         'single tick marks if it is a text or date field
         Dim iDataType, strResult
         iDataType = objColumnDict(strColumnName).DataType

         If iDataType = adDate OR iDataType = adDBTimeStamp _
             OR iDataType = adChar OR iDataType = adVarChar _
             OR iDataType = adLongVarChar OR iDataType = adWChar _
             OR iDataType = adVarWChar OR _
             iDataType = adLongVarWChar then

                strResult = "'" & _
                   Replace(objRS(strColumnName).Value, "'", "''") & "'"
          Else               
                strResult = objRS(strColumnName).Value
          End If
         
         CurrentQuotedValue = strResult
      End Function


      Private Function QuotedValue(strColumnName)
         'Returns the value of a column, surrounding it with
         'single tick marks if it is a text or date field
         Dim iDataType, strResult
         iDataType = objColumnDict(strColumnName).DataType

         If iDataType = adDate OR iDataType = adDBTimeStamp _
             OR iDataType = adChar OR iDataType = adVarChar _
             OR iDataType = adLongVarChar OR iDataType = adWChar _
             OR iDataType = adVarWChar OR _
             iDataType = adLongVarWChar then

                strResult = "'" & _
                   Replace(objColumnDict(strColumnName).Value, "'", "''") & "'"
          Else               
                strResult = objColumnDict(strColumnName).Value
          End If
         
         QuotedValue = strResult
      End Function
      '*****************************************************


      '****************** GET PROPERTIES *******************
      Public Property Get Count()
         Count = objColumnDict.Count
      End Property

      Public Property Get ColumnList()
         Set ColumnList = objColumnDict
      End Property
      '*****************************************************
   End Class
%>